import 'dart:ui';

import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:flutterx_utils/flutterx_utils.dart';

void main() => group('Flutter Extension', () {
      test('testIterable', testIterable);
      test('testBlend', testBlend);
      test('testLightness', testLightness);
      test('testLoggable', testLoggable);
      test('testJsonPath', testJsonPath);
    });

void testIterable() {
  var list = ['Hello', 'World'];
  expect(list.countWhere((item) => item.contains('W')), 1);
  expect(list.count('World'), 1);
  expect(list.any((item) => item.contains('W')), true);
  expect(list.contains('World'), true);
  list.add('World');
  list.add('World');
  list.add('World');
  list.add('World');
  list.add('LoL');
  expect(list.count('World'), 5);
  list = list.toSet().toList();
  expect(list.count('World'), 1);
  list = list.distinct((item) => item.contains('o')).toList();
  expect(list.length, 1);
  list.addIfAbsent('World');
  list.addIfAbsent('World');
  list.addIfAbsent('World');
  list.addIfAbsent('World');
  expect(list.count('World'), 1);
  expect(MyEnum.values.fromName('due'), MyEnum.due);
  expect(MyEnum.values.fromName('sei', fallback: MyEnum.tre), MyEnum.tre);
  expect(MyEnum.values.require((item) => item.toString() == 'MyEnum.due', message: () => 'oh no'), MyEnum.due);
  final json = {'value': 'tre'};
  expect(json.getEnum('value', MyEnum.values), MyEnum.tre);
  final values = [102, 33, 4, 6, 1, 5, -30, 6, 7, 88, 9.0];
  expect(values.closestTo(17), 9);
  expect(values.closestTo(10), 9);
  expect(values.closestTo(7.4), 7);
  expect(values.closestTo(-10), 1);
  expect(values.closestTo(-20), -30);
  expect(values.closestTo(99920), 102);
}

enum MyEnum {
  uno,
  due,
  tre,
}

void testBlend() {
  const red = Color(0xffff0000);
  const yellow = Color(0xffffff00);
  const blue = Color(0xff0000ff);
  expect(Color.lerp(yellow, red, .5), const Color(0xffff7f00));
  expect(Color.lerp(blue, Colors.white, .6), const Color(0xff9999ff));
  expect(Color.lerp(blue, Colors.white, .2), const Color(0xff3333ff));
}

void testLightness() {
  final grey87 = Color.lerp(Colors.white, Colors.black, .87);
  expect(grey87, const Color(0xff212121));
  expect(grey87, equals(Color.lerp(Colors.white, Colors.black, .87)));
  expect(Colors.black.lighten(.13), grey87);
  expect(Colors.white.darken(.87), grey87);
  expect(Colors.white.darken(.5), Colors.black.lighten(.5));
}

void testLoggable() => LoggableTest().start();

class LoggableTest with Loggable {
  void start() => log('Welcome to loggable test', () {
        log('Indented text');
        var result = const Result.success('Hello');
        log(result);
        log(result.resultOrThrow);
        result = Result.wrap(() => throw ArgumentError('Invalid argument'));
        log(result);
      });

  @override
  void logImpl(String message, [Object? error, StackTrace? stack]) =>
      error == null ? debugPrint('$tag:$indent$message') : log('$message\n$error\n$stack');
}

void testJsonPath() {
  final map = {
    'e1': {
      'e2': [
        {'e3a': 123},
        {'e3b': 456},
      ],
      'e4': true
    }
  };

  print('root: ${JsonPath.root.get(map)}');
  print('e1: ${JsonPath('e1').get(map)}');
  print('e2: ${JsonPath.from(const ['e1', 'e2']).get(map)}');
  print('e3a: ${JsonPath.from(const ['e1', 'e2', 0, 'e3a']).get(map)}');
  print('e3b: ${JsonPath.from(const ['e1', 'e2', 1, 'e3b']).get(map)}');
  print('e4: ${JsonPath.from(const ['e1', 'e4']).get(map)}');
  JsonPath.from(const ['e1', 'e4']).set(map, 333);
  print('e4: ${JsonPath.from(const ['e1', 'e4']).get(map)}');
}
